/* mpn_sbpi1_bdiv_r -- schoolbook Hensel division with precomputed inverse,
   returning remainder.

   Contributed to the GNU project by Torbjorn Granlund.

   THE FUNCTION IN THIS FILE IS INTERNAL WITH A MUTABLE INTERFACE.  IT IS ONLY
   SAFE TO REACH IT THROUGH DOCUMENTED INTERFACES.  IN FACT, IT IS ALMOST
   GUARANTEED THAT IT WILL CHANGE OR DISAPPEAR IN A FUTURE GMP RELEASE.

Copyright 2012 Free Software Foundation, Inc.

This file is part of the GNU MP Library.

The GNU MP Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

The GNU MP Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
License for more details.

You should have received a copy of the GNU Lesser General Public License
along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */

#include "gmp.h"
#include "gmp-impl.h"

/* FIXME? This function returns a redc-style remainder, not the type of
   remainder gnerated by mpn_sbpi1_bdiv_qr.  */

mp_limb_t
mpn_sbpi1_bdiv_r (mp_ptr np, mp_size_t nn,
		  mp_srcptr dp, mp_size_t dn, mp_limb_t dinv)
{
  mp_size_t i;
  mp_limb_t q, hi, cy;

  ASSERT (dn > 0);
  ASSERT (nn > dn);
  ASSERT ((dp[0] & 1) != 0);

  cy = 0;

  for (i = nn - dn; i != 0; i--)
    {
      q = dinv * np[0];
      hi = mpn_addmul_1 (np, dp, dn, q);
      hi += cy;
      cy = hi < cy;
      hi += np[dn];
      cy += hi < np[dn];
      np[dn] = hi;
      np++;
    }

  return cy;
}
